@@ -1,20 +1,18 @@
 #' @title The Fitting Function of \code{GPM} Package
-#' @description Fits a Gaussian process (GP) to a set of simulation data as described in \code{reference 1}. Both the inputs and outputs can be multi-dimensional. The outputs can be noisy.
+#' @description Fits a Gaussian process (GP) to a set of simulation data as described in \code{reference 1}. Both the inputs and outputs can be multi-dimensional. The outputs can be noisy (the noise variance is assumed to be homogeneous).
 #' @param X Matrix containing the training (aka design or input) data points. The rows and columns of \code{X} denote individual observation settings and input dimension, respectively.
 #' @param Y Matrix containing the output (aka response) data points. The rows and columns of \code{Y} denote individual observation responses and output dimension, respectively.
-#' @param CorrType The type of the correlation function of the GP model. Choices include \code{'G'} (default), \code{'PE'}, \code{'LBG'}, and \code{'LB'}. See the \code{references} for the details. For smooth (or analytic) functions, choose either
+#' @param CorrType The type of the correlation function of the GP model. Choices include \code{'G'} (default), \code{'PE'}, \code{'LBG'}, and \code{'LB'}. See the \code{references} for the details. For smooth (or analytic) functions, choose either \code{'G'} or \code{'LBG'}.
 #' @param Eps A vector containing the smallest eigen value(s) that the correlation matrix is allowed to have. The elements of Eps must be in [0, 1] and sorted in a descending order.
-#' @param Nopt The number of times the log-likelihood function is optimized when Eps[1] is used to constraint the smallest eigen value that the correlation matrix is allowed to have. Higher \code{Nopt} will increase fitting time as well as the chances of finding the global optimum.
+#' @param Nopt The number of times the log-likelihood function is optimized when Eps[1] is used to constrain the smallest eigen value that the correlation matrix is allowed to have. Higher \code{Nopt} will increase fitting time as well as the chances of finding the global optimum.
 #' @param TraceIt Non-negative integer. If positive, tracing information on the progress of the optimization is \strong{printed}. There are six levels of tracing (see \code{\link{optim}}) and higher values will produce more tracing information.
 #' @param MaxIter Maximum number of iterations allowed for each optimization (see \code{\link{optim}}).
 #' @param Seed An integer for the random number generator. Use this to make the results reproducible.
 #' @param LowerBound,UpperBound To estimate the scale (aka roughness) parameters of the correlation function, the feasible range should be defined. \code{LowerBound} and \code{UpperBound} are vectors determining, resepectively, the lower and upper bounds and their length depends on the parametric form of the correlation function (see \code{reference 1} for the details).
-#' @param Criterion Either \code{LOOCV} or \code{LogL}. Criterion used to determine the most optimum constraint (i.e. an element of \code{Eps}).
 #' @param StopFlag \code{Flag} indicating whether the optimization must be stopped if the constraint on the correlation matrix is inactive for two consecuitvie elements of \code{Eps}.
 #' @param Progress \code{Flag} indicating if the fitting process should be summarized. Set it to \code{!=0} to turn it on.
 #' @import lhs
 #' @import randtoolbox
-#'
 #' @return Model A list containing the following components:
 #' \itemize{
 #' \item{\code{CovFunc}} {A list containing the type and estimated parameters of the correlation function.}
@@ -26,8 +24,8 @@
 #'
 #' @references
 #' \enumerate{
-#' \item Bostanabad, R., Kearney, T., Tao, S. Y., Apley, D. W. & Chen, W. (2018) Leveraging the nugget parameter for efficient Gaussian process modeling. International Journal for Numerical Methods in Engineering, 114, 501-516.
-#' \item M. Plumlee, D.W. Apley (2016). Lifted Brownian kriging models, Technometrics.
+#' \item Bostanabad, R., Kearney, T., Tao, S., Apley, D. W. & Chen, W. Leveraging the nugget parameter for efficient Gaussian process modeling. International Journal for Numerical Methods in Engineering, doi:10.1002/nme.5751.
+#' \item Plumlee, M. & Apley, D. W. (2017) Lifted Brownian kriging models. Technometrics, 59, 165-177.
 #' }
 #' @export
 #' @seealso
@@ -38,46 +36,47 @@
 #' # 1D example: Fit a model (with default settings) and evaluate the performance
 #' # by computing the root mean squared error (RMSE) in prediction.
 #' library(lhs)
-#' X <- 5*maximinLHS(10, 1)
-#' Y <- 2*sin(2*X) + log(X+1)
+#' X <- 5*maximinLHS(8, 1)
+#' Y <- 2*sin(X) + log(X+1)
 #' M <- Fit(X, Y)
-#' XF <- matrix(seq(0, 5, length.out = 100), 100, 1)
+#' XF <- matrix(seq(0, 5, length.out = 50), 50, 1)
 #' YF <- Predict(XF, M)
-#' RMSE <- sqrt(mean((YF$YF - (2*sin(2*XF) + log(XF+1)))^2))
+#' RMSE <- sqrt(mean((YF$YF - (2*sin(XF) + log(XF+1)))^2))
+#'
+#' \dontrun{
 #'
 #' # 1D example: Fit a model, evaluate the performance, and plot the response
 #' # along with 95% prediction interval
-#' X <- 10*maximinLHS(25, 1) - 5
-#' Y <- X*cos(pi*X)
+#' X <- 10*maximinLHS(18, 1) - 5
+#' Y <- X*cos(X)
 #' M <- Fit(X, Y)
-#' XF <- matrix(seq(-5, 5, length.out = 100), 100, 1)
+#' XF <- matrix(seq(-5, 5, length.out = 50), 50, 1)
 #' YF <- Predict(XF, M)
-#' RMSE <- sqrt(mean((YF$YF - (XF*cos(pi*XF)))^2))
-#' Draw(M, 1, res = 100)
-#'
-#' \dontrun{
+#' RMSE <- sqrt(mean((YF$YF - (XF*cos(XF)))^2))
+#' Draw(M, 1, res = 15)
+#' 
 #' # 2D example: Fit a model, evaluate the performance, and plot the response
 #' # surface along with 95% prediction interval
-#' X <- 2*maximinLHS(20, 2) - 1
+#' X <- 2*maximinLHS(10, 2) - 1
 #' Y <- X[, 1]^2 + X[, 2]^2
 #' M <- Fit(X, Y, CorrType = "PE")
 #' XF <- 2*maximinLHS(100, 2) - 1
 #' YF <- Predict(XF, M)
 #' RMSE <- sqrt(mean((YF$YF - (XF[, 1]^2 + XF[, 2]^2))^2))
 #' library(lattice)
-#' Draw(M, c(1, 1), res = 20, PI95=1)
+#' Draw(M, c(1, 1), res = 15, PI95=1)
 #'
 #' # 2D example: Plot the previous model wrt X1 in the [-2, 2]
 #' # interval with X2=1
-#' Draw(M, c(1, 0), LB = -2, UB = 2, res = 20, PI95=1)
+#' Draw(M, c(1, 0), LB = -2, UB = 2, res = 15, PI95=1)
 #'
 #' # 3D example: Compare the performance of Gaussian ("G") and lifted Browninan
 #' # with Gamma=1 ("LBG")
-#' X <- 2*maximinLHS(50, 3) - 1
+#' X <- 2*maximinLHS(30, 3) - 1
 #' Y <- cos(X[, 1]^2) + 2*sin(X[, 2]^2) + X[, 3]^2
 #' M_G <- Fit(X, Y)
 #' M_LBG <- Fit(X, Y, CorrType = "LBG")
-#' XF <- 2*maximinLHS(1000, 3) - 1
+#' XF <- 2*maximinLHS(50, 3) - 1
 #' YF_G <- Predict(XF, M_G)
 #' YF_LBG <- Predict(XF, M_LBG)
 #' RMSE_G <- sqrt(mean((YF_G$YF - (cos(XF[, 1]^2) + 2*sin(XF[, 2]^2) + XF[, 3]^2))^2))
@@ -89,7 +88,7 @@
 
 Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt = 0,
                  MaxIter = 100, Seed = 1, LowerBound = NULL, UpperBound = NULL,
-                 Criterion = 'LOOCV', StopFlag = 0, Progress = 0) {
+                 StopFlag = 0, Progress = 0) {
 
   set.seed(Seed)
 
@@ -97,11 +96,8 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   if (Progress != 0){
     cat('*** Checking the inputs ...\n')
   }
-  if (missing(X)){
-    stop('     X must be provided.')
-  }
-  if (missing(Y)){
-    stop('     Y must be provided.')
+  if (missing(X) || missing(Y)){
+    stop('     X and Y must be provided.')
   }
   if (!all(is.finite(X)) || !is.numeric(X)){
     stop('     All the elements of X must be finite numbers.')
@@ -121,7 +117,7 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   if (is.character(CorrType)){
     CorrType <- toupper(CorrType)
   } else {
-    stop('CorrType should be a character (options are: "G", "PE", "LB", and "LBG")')
+    stop('     CorrType should be a character (options are: "G", "PE", "LB", and "LBG")')
   }
   if (CorrType!= 'G' && CorrType!= 'PE' && CorrType!= 'LBG' && CorrType!='LB'){
     stop('     The type of the correlation function is not determined correctly. Supported functions types are "G", "PE", "LB", and "LBG".')
@@ -132,15 +128,8 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   if (any(diff(Eps) > 0)){
     stop('The elements of Eps should be in a descending order.')
   }
-  if (is.character(Criterion)){
-    if(Criterion != 'LOOCV' && Criterion != 'NLogL'){
-      stop('Criterion should be either LOOCV or NLogL.')
-    }
-  } else{
-    stop('Criterion should be a string (either LOOCV or NLogL).')
-  }
 
-  k <- nrow(X); p <- ncol(X); q <- ncol(Y);
+  n <- nrow(X); dx <- ncol(X); dy <- ncol(Y);
 
 
   Omega_min_default <- -2
@@ -151,52 +140,48 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   Gamma_max_default <- 1
 
   if (is.null(LowerBound)){
-    if (Progress != 0){
-      cat('     The default values are used for initializing the lower bounds on hyper-parameters.\n')
-    }
-    if (CorrType == 'G') {LowerBound = rep(x = Omega_min_default, times = p)}
-    else if (CorrType == 'PE') {LowerBound = c(rep(x = Omega_min_default, times = p), 1)}
-    else if (CorrType == 'LBG') {LowerBound = c(rep(x = Omega_min_default, times = p), Zeta_min_default)}
-    else if (CorrType == 'LB') {LowerBound = c(rep(x = Omega_min_default, times = p), Zeta_min_default, Gamma_min_default)}
+    LB_Def <- 1
+    if (CorrType == 'G') {LowerBound = rep(x = Omega_min_default, times = dx)}
+    else if (CorrType == 'PE') {LowerBound = c(rep(x = Omega_min_default, times = dx), 1)}
+    else if (CorrType == 'LBG') {LowerBound = c(rep(x = Omega_min_default, times = dx), Zeta_min_default)}
+    else if (CorrType == 'LB') {LowerBound = c(rep(x = Omega_min_default, times = dx), Zeta_min_default, Gamma_min_default)}
   } else {
+    LB_Def <- 0
     if (!is.vector(LowerBound)) LowerBound = as.vector(LowerBound)
-    if (CorrType == 'G' && length(LowerBound) != p){
-      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(p)))}
-    else if (CorrType == 'PE' && length(LowerBound) != (1 + p)){
-      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(p + 1)))}
-    else if (CorrType == 'LBG' && length(LowerBound) != (1 + p)){
-      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(p + 1)))}
-    else if (CorrType == 'LB' && length(LowerBound) != (2 + p)){
-      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(p + 2)))}
-    if (any(LowerBound[1:p] < -10)){
-      warning(paste('The first ', toString(p), ' values in LowerBound are recommended to be larger than -10.'))
-    }
+    if (CorrType == 'G' && length(LowerBound) != dx){
+      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(dx)))}
+    else if (CorrType == 'PE' && length(LowerBound) != (1 + dx)){
+      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(dx + 1)))}
+    else if (CorrType == 'LBG' && length(LowerBound) != (1 + dx)){
+      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(dx + 1)))}
+    else if (CorrType == 'LB' && length(LowerBound) != (2 + dx)){
+      stop(paste('     The size of the provided LowerBound is incorrect! It should be', toString(dx + 2)))}
   }
 
   if (is.null(UpperBound)){
-    if (Progress != 0){
-      cat('     The default values are used for initializing the upper bounds on hyper-parameters.\n')
-    }
-    if (CorrType == 'G') {UpperBound = rep(x = Omega_max_default, times = p)}
-    else if (CorrType == 'PE') {UpperBound = c(rep(x = Omega_max_default, times = p), 2)}
-    else if (CorrType == 'LBG') {UpperBound = c(rep(x = Omega_max_default, times = p), Zeta_max_default)}
-    else if (CorrType == 'LB') {UpperBound = c(rep(x = Omega_max_default, times = p), Zeta_max_default, Gamma_max_default)}
+    UB_Def <- 1
+    if (CorrType == 'G') {UpperBound = rep(x = Omega_max_default, times = dx)}
+    else if (CorrType == 'PE') {UpperBound = c(rep(x = Omega_max_default, times = dx), 2)}
+    else if (CorrType == 'LBG') {UpperBound = c(rep(x = Omega_max_default, times = dx), Zeta_max_default)}
+    else if (CorrType == 'LB') {UpperBound = c(rep(x = Omega_max_default, times = dx), Zeta_max_default, Gamma_max_default)}
   } else {
+    UB_Def <- 0
     if (!is.vector(UpperBound)) UpperBound = as.vector(UpperBound)
-    if (CorrType == 'G' && length(UpperBound) != p){
-      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(p)))}
-    else if (CorrType == 'PE' && length(UpperBound) != (1 + p)){
-      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(p + 1)))}
-    else if (CorrType == 'LBG' && length(UpperBound) != (1 + p)){
-      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(p + 1)))}
-    else if (CorrType == 'LB' && length(UpperBound) != (2 + p)){
-      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(p + 2)))}
+    if (CorrType == 'G' && length(UpperBound) != dx){
+      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(dx)))}
+    else if (CorrType == 'PE' && length(UpperBound) != (1 + dx)){
+      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(dx + 1)))}
+    else if (CorrType == 'LBG' && length(UpperBound) != (1 + dx)){
+      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(dx + 1)))}
+    else if (CorrType == 'LB' && length(UpperBound) != (2 + dx)){
+      stop(paste('     The size of the provided UpperBound is incorrect! It should be', toString(dx + 2)))}
   }
-  if (CorrType == 'PE' && (LowerBound[p + 1] < 1 || UpperBound[p+ 1] > 2)){
+  if (CorrType == 'PE' && (LowerBound[dx + 1] < 1 || UpperBound[dx+ 1] > 2)){
     stop(paste('     The exponent in PE should be between ', toString(c(1, 2)), ' (inclusive).'))
-  } else if (CorrType == 'LBG' && (LowerBound[p + 1] <= 0 || UpperBound[p+ 1] > Zeta_max_default)){
+  } else if (CorrType == 'LBG' && (LowerBound[dx + 1] <= 0 || UpperBound[dx+ 1] > Zeta_max_default)){
     stop(paste('     Beta in LBG should be between ', toString(c(0, 1)), ' (exclusive).'))
-  } else if (CorrType == 'LB' && (LowerBound[p+1]<=0 || UpperBound[p+1]>Zeta_max_default || LowerBound[p+2]<Gamma_min_default || UpperBound[p+2]>Gamma_max_default)){
+  } else if (CorrType == 'LB' && (LowerBound[dx+1]<=0 || UpperBound[dx+1]>Zeta_max_default || 
+                  LowerBound[dx+2]<Gamma_min_default || UpperBound[dx+2]>Gamma_max_default)){
     stop('     The provided range for Beta and/or Gamma is not acceptable.')
   }
 
@@ -217,11 +202,11 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   if (CorrType == 'LBG' || CorrType == 'LB'){
     XN0 <- XN[1, ]
     YN0 <- YN[1, ]
-    XN <- t(t(XN) - XN0)[2:k, ]
-    YN <- t(t(YN) - YN0)[2:k, , drop = FALSE]
-    k <- k - 1
+    XN <- t(t(XN) - XN0)[2:n, ]
+    YN <- t(t(YN) - YN0)[2:n, , drop = FALSE]
+    n <- n - 1
   }
-  Fn <- matrix(data = 1, nrow = k, ncol = 1)
+  Fn <- matrix(data = 1, nrow = n, ncol = 1)
 
   ptm <- proc.time()
 
@@ -231,7 +216,8 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   N_unique_local_opt <- matrix(0, Tries, 1)
   OptimHist <- NULL
   OptimHist$Var_int[[1]] <- t(t(A)*(UpperBound - LowerBound) + LowerBound)
-  CTRL <- c(trace=0, maxit = MaxIter,  REPORT = 1, lmm = 15, factr = 1e6, pgtol = 1e-8)
+  CTRL <- c(trace = TraceIt, maxit = MaxIter,  REPORT = 1, lmm = 15, factr = 1e6, pgtol = 1e-8)
+
   for (i in 1:Tries){
     Var_int <- OptimHist$Var_int[[i]]
     Eps_i <- Eps[i]
@@ -239,26 +225,25 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
     Var <- matrix(0, Nopt_i, N_hyperC)
     Obj_Fun <- matrix(0, Nopt_i, 1)
     Exit_Flag <- matrix(0, Nopt_i, 1)
-    if (i > 1){
-      LowerBound[1: p] <- -10
-      UpperBound[1: p] <- 4
+    if (LB_Def == 1 && i > 1){
+      LowerBound[1: dx] <- -10
+    }
+    if (UB_Def == 1 && i > 1){
+      UpperBound[1: dx] <- 4
     }
     for (j in 1: Nopt_i){
-      temp <- stats::optim(Var_int[j, ], NLogL, gr = NULL, XN, YN, CorrType, Eps_i, Fn, k, q,
+      temp <- stats::optim(Var_int[j, ], NLogL, gr = NULL, XN, YN, CorrType, Eps_i, Fn, n, dy,
                     method = 'L-BFGS-B', lower = LowerBound, upper = UpperBound, control = CTRL)
       Var[j, ] <- temp$par
       Obj_Fun[j] <- temp$value
       Exit_Flag[j] <- temp$convergence
     }
-
-
-
+    
     ID = sort(Obj_Fun, index.return=TRUE)$ix
     Obj_Fun <- as.matrix(Obj_Fun[ID, drop = FALSE])
     Exit_Flag <- as.matrix(Exit_Flag[ID, drop = FALSE])
     Var <- as.matrix(Var[ID, , drop = FALSE])
 
-
     if (Progress != 0){
       Table <- cbind(apply(round(Var, 3), 1, toString), round(Obj_Fun, 2))
       colnames(Table)<- c('Estimated Hyperparameters', ' NLogL')
@@ -273,7 +258,7 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
     Exit_unique <- as.matrix(as.data.frame(Exit_Flag)[c(rownames(Obj_Fun_unique)), ])
     Var_unique <- as.matrix(as.data.frame(Var)[c(rownames(Obj_Fun_unique)), ])
 
-    if (Progress != 0 && Nopt_i > 0){
+    if (Progress != 0 && Nopt_i > 1){
       Table <- cbind(apply(round(Var_unique, 3), 1, toString), round(Obj_Fun_unique, 2))
       colnames(Table)<- c('Estimated Hyperparameters', ' NLogL')
       Table <- as.table(Table)
@@ -282,32 +267,14 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
       print(Table, row.names = FALSE)
     }
 
-    R <- CorrMat(XN, XN, CorrType, Var_unique[1, ])
-    Raw_MinEig <- sort(eigen(R, symmetric = TRUE, only.values = TRUE)$values)[1]
+    R <- CorrMat_Sym(XN, CorrType, Var_unique[1, ])
+    Raw_MinEig <- Eigen(R)[1] #sort(eigen(R, symmetric = TRUE, only.values = TRUE)$values)[1]
     if (Raw_MinEig < Eps[i]){
       Nug_opt <- Eps[i] - Raw_MinEig;
-      R <- R + diag(x = 1, k, k)*(Eps[i] - Raw_MinEig)
+      R <- R + diag(x = 1, n, n)*(Eps[i] - Raw_MinEig)
     } else {
       Nug_opt <- 0
     }
-    L <- t(chol(R))
-    inv_L <- solve(L)
-    Rinv_YN <- solve(t(L), solve(L, YN))
-    LOOCV <- matrix(0, q, 1)
-    if (CorrType == 'PE' || CorrType=='G'){
-      Rinv_Fn <- solve(t(L), solve(L, Fn))
-      FnTRinvFn <- t(Fn)%*%Rinv_Fn
-      B <- t(Fn)%*%Rinv_YN/FnTRinvFn[1]
-      for (ii in 1:q){
-        temp <- solve(diag(diag(t(inv_L)%*%inv_L)), Rinv_YN[, ii, drop = FALSE] - Rinv_Fn%*%B[ii])
-        LOOCV[ii] <- mean(abs(temp))
-      }
-    } else {
-      for (ii in 1:q){
-        temp <- solve(diag(diag(t(inv_L)%*%inv_L)), Rinv_YN[, ii, drop = FALSE])
-        LOOCV[ii] <- mean(abs(temp))
-      }
-    }
 
     OptimHist$Nug_opt[[i]] <- Nug_opt
     OptimHist$Raw_MinEig[[i]] <-  Raw_MinEig
@@ -316,7 +283,6 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
     OptimHist$Obj_Fun[[i]] <-  Obj_Fun
     OptimHist$Obj_Fun_unique[[i]] <- Obj_Fun_unique
     OptimHist$Obj_Fun_best[[i]] <-  Obj_Fun[1]
-    OptimHist$LOOCV[[i]] <- LOOCV
     OptimHist$Exit_unique[[i]] <- Exit_unique
 
     if (i > 1 && (StopFlag != 0) && (OptimHist$Nug_opt[[i]] == OptimHist$Nug_opt[[i-1]])){
@@ -330,70 +296,60 @@ Fit <-  function(X, Y, CorrType = 'G', Eps = 10^(seq(-1, -12)),Nopt = 5,TraceIt
   Cost <- proc.time()[3] - ptm[3]
   # Organize the optimization results
   #cat('*** Summary of the optimization results (sorted and rounded):\n')
-  if (Criterion == 'LOOCV'){
-    ID <- which.min(lapply(OptimHist$LOOCV, sum))
-  } else {
-    ID <- which.min(OptimHist$Obj_Fun_best)
-  }
+  ID <- which.min(OptimHist$Obj_Fun_best)
   w <- as.matrix(OptimHist$Var[[ID]][1, ])
 
   ## Post-processing
-  R <- CorrMat(XN, XN, CorrType, w)
-  Raw_MinEig <- sort(eigen(R, symmetric = TRUE, only.values = TRUE)$values)[1]
+  R <- CorrMat_Sym(XN, CorrType, w)
+  Raw_MinEig <- Eigen(R) #sort(eigen(R, symmetric = TRUE, only.values = TRUE)$values)[1]
   if (Raw_MinEig < Eps[ID]){
     Nug_opt <- Eps[ID] - Raw_MinEig;
     if (Progress != 0){
-      cat('\n*** The best model is fitted via Eps = ', toString(Eps[ID]), '. A nugget of ', Nug_opt, ' has been used.')
+      cat('\n*** The best model is fitted via Eps = ', toString(Eps[ID]), 
+          '. A nugget of ', Nug_opt, ' has been used.')
     }
-    R <- R + diag(x = 1, k, k)*(Eps[ID] - Raw_MinEig)
+    R <- R + diag(x = 1, n, n)*(Eps[ID] - Raw_MinEig)
   } else {
     Nug_opt <- 0
   }
 
-  L <- t(chol(R))
-  inv_L <- solve(L)
-  LOOCV <- matrix(0, q, 1)
-  Rinv_YN <- solve(t(L), solve(L, YN))
+  L <- LowerChol(R)
+  Rinv_YN <- CppSolve(t(L), CppSolve(L, YN))
   if (CorrType == 'PE' || CorrType=='G'){
-    Rinv_Fn <- solve(t(L), solve(L, Fn))
+    Rinv_Fn <- CppSolve(t(L), CppSolve(L, Fn))
     FnTRinvFn <- t(Fn)%*%Rinv_Fn
     B <- t(Fn)%*%Rinv_YN/FnTRinvFn[1]
     temp <- YN - Fn%*%B
-    Sigma2 <- t(temp)%*%solve(t(L), solve(L, temp))/k
+    Sigma2 <- t(temp)%*%CppSolve(t(L), CppSolve(L, temp))/n
     if (CorrType=='G'){
       Theta <- w; Power <- 2;
     } else {
-      Theta <- w[1:p]; Power <- w[p+1]
+      Theta <- w[1:dx]; Power <- w[dx+1]
       if (Power > 1.999){
         Power <- 2
       }
     }
-    Parameters <- list(FnTRinvFn = FnTRinvFn[1], B = B, Sigma2 = Sigma2, Rinv_Fn = Rinv_Fn,
-                      Rinv_YN = Rinv_YN, Theta = Theta, Power = Power)
-    for (i in 1:q){
-      temp <- solve(diag(diag(t(inv_L)%*%inv_L)), Rinv_YN[, i] - Rinv_Fn%*%B[i])
-      LOOCV[i] <- mean(abs(temp))
-    }
+    Parameters <- list('FnTRinvFn' = FnTRinvFn[1], 'B' = B, 'Sigma2' = Sigma2, 'Rinv_Fn' = Rinv_Fn,
+                      'Rinv_YN' = Rinv_YN, 'Theta' = Theta, 'Power' = Power)
   } else {
-    Alpha <- t(YN)%*%Rinv_YN/k
+    Alpha <- t(YN)%*%Rinv_YN/n
     if (CorrType == 'LBG'){
-      A <- w[1:p]; Beta <- w[p+1]; Gamma <- 1
+      A <- w[1:dx]; Beta <- w[dx+1]; Gamma <- 1
     } else {
-      A <- w[1:p]; Beta <- w[p+1]; Gamma <- w[p+2]
+      A <- w[1:dx]; Beta <- w[dx+1]; Gamma <- w[dx+2]
       if (Gamma>0.999) Gamma <- 1
     }
-    Parameters <- list(XN0 = XN0, YN0 = YN0, Alpha = Alpha, Rinv_YN = Rinv_YN,
-                      A = A, Beta = Beta, Gamma = Gamma)
-    for (i in 1:q){
-      temp <- solve(diag(diag(t(inv_L)%*%inv_L)), Rinv_YN[, i])
-      LOOCV[i] <- mean(abs(temp))
-    }
+    Parameters <- list('XN0' = XN0, 'YN0' = YN0, 'Alpha' = Alpha, 'Rinv_YN' = Rinv_YN,
+                      'A' = A, 'Beta' = Beta, 'Gamma' = Gamma)
   }
   ## Save the results
   Model <- NULL
-  Model$CovFunc <- list('CorrType' = CorrType, 'Parameters' = Parameters, 'LowerBound' = LowerBound, 'UpperBound' = UpperBound)
-  Model$Data <- list('XN' = XN, 'k' = k, 'Xmin' = Xmin, 'Xmax' = Xmax,  'YN' = YN, 'q' = q, 'Yrange' = Yrange, 'Ymin' = Ymin)
-  Model$Details <- list('Fn' = Fn, 'L' = L, 'Raw_MinEig' = Raw_MinEig, 'Nug_opt' = Nug_opt, 'LOOCV' = LOOCV, 'Cost' = Cost, 'MinNLogL' = OptimHist$Obj_Fun_best[ID])
+  Model$CovFunc <- list('CorrType' = CorrType, 'Parameters' = Parameters, 
+                        'LowerBound' = LowerBound, 'UpperBound' = UpperBound)
+  Model$Data <- list('XN' = XN, 'n' = n, 'Xmin' = Xmin, 'Xmax' = Xmax,  'YN' = YN, 
+                     'dy' = dy, 'Yrange' = Yrange, 'Ymin' = Ymin)
+  Model$Details <- list('Fn' = Fn, 'L' = L, 'Raw_MinEig' = Raw_MinEig, 'Nug_opt' = Nug_opt, 
+                        'Cost' = Cost, 'MinNLogL' = OptimHist$Obj_Fun_best[ID])
   Model$OptimHist <- OptimHist
   Model$Setting <- Setting
   class(Model) <- 'GPM'
